// https://circuitdigest.com/microcontroller-projects/i2c-communication-with-pic-microcontroller-pic16f877a
// https://www.mikroe.com/ebooks/pic-microcontrollers-programming-in-c/serial-communication-modules
#define _XTAL_FREQ 16000000
#define WRITE 0x90
//#define WRITE 0x48

unsigned char I2C_Read(unsigned char ack);
void I2C_Write(unsigned char datum);
void I2C_Stop();
void I2C_Start();
void I2C_Restart();
void I2C_Hold();
void I2C_Initialize(const unsigned long freq);

void ds1621Init();
int readTemp();
unsigned char readCntr();

void main() {

     int T;

     I2C_Initialize(100000);
     ds1621Init();

     while(1) {
          T = readTemp();
//          T = (int)readCntr();
          delay_ms(750);
     }//while

     return;
}//main

void I2C_Initialize(const unsigned long freq) {//Begin IIC as master

//     TRISC.F3 = 1; TRISC.F4 = 1;  // Set SDA and SCL pins as input pins
     TRISC |= 0x18;//0b00011000;
     TRISA.F0 = 0;

     SSPCON  = 0b00101000;    //pg84/234
     SSPCON2 = 0b00000000;    //pg85/234

     SSPADD = _XTAL_FREQ/(4*freq) - 1; //Setting Clock Speed pg99/234
//     SSPADD = 0x27;
     SSPSTAT = 0b00000000;    //pg83/234

     return;
}//I2C_Initialize

void I2C_Hold() {

// https://simulide.com/p/forum/topic/does-i2c-work-for-any-of-the-pic16f-mcus/
#ifndef SIMULIDE
    delay_us(100);
#else
    while ((SSPCON2 & 0x1F ) || (SSPSTAT & 0x04)); // Wait for any pending transfer
#endif

    return;
}//I2C_Hold

void I2C_Start() {

     I2C_Hold();  //Hold the program is I2C is busy
     SSPCON2.SEN = 1;     //Begin IIC pg85/234
     while(SSPCON2.SEN);

     return;
}//I2C_Start

void I2C_Restart() {

     I2C_Hold();
     SSPCON2.RSEN = 1;
     while(SSPCON2.RSEN);

     return;
}//I2C_Restart

void I2C_Stop() {

     I2C_Hold(); //Hold the program is I2C is busy
     SSPCON2.PEN = 1;    //End IIC pg85/234
     while(SSPCON2.PEN);

     return;
}//I2C_Stop

void I2C_Write(unsigned char datum) {

     I2C_Hold(); // Hold the program is I2C is busy
     SSPBUF = datum; //pg82/234
     while(SSPSTAT.BF);

     I2C_Hold();
     // Check if acknowledge bit has arrived from slave
     if (SSPCON2.ACKSTAT) PORTA.F0 = ~PORTA.F0;

     return;
}//I2C_Write

unsigned char I2C_Read(unsigned char ack) {

         unsigned char incoming;

         I2C_Hold();
         SSPCON2.RCEN = 1;
         while(!SSPSTAT.BF);

         I2C_Hold();
         incoming = SSPBUF;         // get the data saved in SSPBUF

         I2C_Hold();
         SSPCON2.ACKDT = (ack)?0:1; // check if ack bit received
         SSPCON2.ACKEN = 1;         // pg 85/234

         return incoming;
}//I2C_Read

void ds1621Init() {

     // initialize DS1621 sensor
     I2C_Start();
     I2C_Write(WRITE);  // Write mode
     I2C_Write(0xAC);   // send configuration register address (Access Config)
     I2C_Write(0x01);   // performs either continuous (0x00) or one-shot (0x01) conversion
//     I2C_Stop();

     I2C_Restart();
     I2C_Write(WRITE);  // Write mode
     I2C_Write(0xEE);   // send start temperature conversion command
     I2C_Stop();

     return;
}//ds1621Init

int readTemp() {

     unsigned char tH, tL;

     I2C_Start();

     I2C_Write(WRITE);        // Write mode
     I2C_Write(0xAA);         // Access temperature register
//     I2C_Stop();

//     I2C_Start();
     I2C_Restart();
     I2C_Write(WRITE | 0x01); // Read mode
     tH = I2C_Read(1);        // Read temperature high, ack
     tL = I2C_Read(0);        // Read temperature low, nack

     I2C_Stop();

     return (tH<<8) | tL;
}//readTemp

unsigned char readCntr() {

     unsigned char out;

     I2C_Start();

     I2C_Write(WRITE);        // Write mode
     I2C_Write(0xA8);         // Access counter register
//     I2C_Stop();

//     I2C_Start();
     I2C_Restart();
     I2C_Write(WRITE | 0x01); // Read mode
     out = I2C_Read(0);       // Read counter register, nack

     I2C_Stop();

     return out;
}//readCntr